/* -*- mode: c++ -*- */
//error.h
// manejo elemental de errores de ejecucin
#ifndef __ERROR__
#define __ERROR__
#include <iostream>  
#include <string>  
using namespace std;

/**
 *\defgroup error Manejo de errores en Calnum
 */

/// Cdigos de error en Calnum
/** enumeracin annima cuyo cometido es asociar
 * nombres mnemotcnicos a una serie
 * de nmeros enteros que representan posibles errores de ejecucin
 *\ingroup error   
 */
enum { 
    EXITO    = 0, 
    FALLO    = -1,
    ITERANDO = -2,  /* todava iterando */
    CEDOM     = 1,   /* error de dominio, e.g sqrt(-1) */
    CERANGO   = 2,   /* rango incorrecto */
    CEPTR     = 3,   /* puntero no vlido */
    CEINVAL   = 4,   /* argumento no vlido  */
    CEFALLO   = 5,   /* fallo genrico (no sabemos bien qu ha pasado) */
    CEFACTOR  = 6,   /* error de factorizacin */
    CESANITY  = 7,   /* error de cordura (sanity en ingls). Caso desesperado*/
    CENOMEM   = 8,   /* "No hay Memoria", error al pedir recursos en new */
    CEFUNC    = 9,   /* problemas con la funcin escrita por el usuario */
    CENOCTRL  = 10,  /* el proceso iterativo esta fuera de control*/ 
    CEMAXITER = 11,  /* excedido el nmero mximo de iteraciones */
    CEZERODIV = 12,  /* intento de dividir por cero */
    CEMALTOL  = 13,  /* tolerancia proporcionada por el usuario invlida */
    CETOL     = 14,  /* el proceso no alcanza la tolerancia especificada */
    CEUNDRFLW = 15,  /* desbordamiento negativo de un real o entero*/
    CEOVRFLW  = 16,  /* desbordamiento positivo de un real o entero */
    CEPRECI   = 17,  /* prdida de precisin */
    CEREDON   = 18,  /* fallo debido a error de redondeo*/
    CEMALDIM  = 19,  /* problema con la dimensin de una matriz o vector */
    CENOCUAD  = 20,  /* matriz no cuadrada */
    CESING    = 21,  /* singularidad*/
    CEDIVERGE = 22,  /* integral o serie divergente */
    CENOPROG  = 23,  /* iteracin estancada*/
    CENOINI   = 24,  /* Error de incializacin*/
    CENOFCHRO = 25,  /* Error al abrir un fichero*/
    CEPROCESO = 26,  /* Error en el proceso*/
    CNOACOTADO = 27,  /* Error en el proceso*/
} ;

/// Funcin error 
/**
 * Escribe un mensaje por la pantalla y detiene la ejecucin
 * enviando al sistema un cdigo de error
 *\ingroup error
 */
inline void error (string mensaje, int codigo)
{
    std::cerr << mensaje << endl;
    switch(codigo) {
    case FALLO: 
	std::cerr << "Fallo al ejecutar" << endl;
	exit(FALLO);
	break;
    case CEDOM: 
	std::cerr << "Error de dominio, raz cuadrada de un nmero negativo" 
	     << "o similar" << endl;
	exit(CEDOM);
	break;
    case CERANGO: 
	std::cerr << "Rango incorrecto" <<endl; 
	exit(CERANGO);
	break;
    case CEPTR: 
	std::cerr << "Puntero no vlido" << endl;
	exit(CEPTR);
	break;
    case CEINVAL: 
	std::cerr << "Argumento no vlido" << endl;
	exit(CEINVAL);
	break;
    case CENOMEM: 
	std::cerr << "No hay bastante memoria" << endl;
	exit(CENOMEM);
	break;
    case CEFUNC: 
	std::cerr << "Hay un problema con la funcin del usuario" << endl;
	exit(CEFUNC);
	break;
    case CEMAXITER: 
	std::cerr << "Se ha excedido el nmero mximo de iteraciones" << endl;
	exit(CEMAXITER);
	break;
    case CEZERODIV: 
	std::cerr << "Intento de dividir por cero" << endl;
	exit(CEZERODIV);
	break;
    case CEMALTOL: 
	std::cerr << "Tolerancia invlida" << endl;
	exit(CEMALTOL);
	break;
    case CETOL: 
	std::cerr << "El proceso no alcanza la tolerancia especificada" << endl;
	exit(CETOL);
	break;
    case CEUNDRFLW: 
	std::cerr << "Error de underflow" << endl;
	exit(CEUNDRFLW);
    case CEOVRFLW: 
	std::cerr << "Error de overflow" << endl;
	exit(CEOVRFLW);
	break;
    case CEPRECI: 
	std::cerr << "Perdida de precisin" << endl;
	exit(CEPRECI);
	break;
    case CEREDON: 
	std::cerr << "Error de Redondeo" << endl;
	exit(CEREDON);
	break;
    case CEMALDIM: 
	std::cerr << "Problema con la dimensin de una matriz o vector" << endl;
	exit(CEMALDIM);
	break;
    case CENOCUAD: 
	std::cerr << "Error, la matriz no es cuadrada" << endl;
	exit(CENOCUAD);
	break;
    case CESING: 
	std::cerr << "Se ha detectado una singularidad" << endl;
	exit(CESING);
	break;
    case CEDIVERGE: 
	std::cerr << "Integral o serie divergente" << endl;
	exit(CEDIVERGE);
	break;
    case CENOPROG: 
	std::cerr << "Iteracin estancada" << endl;
	exit(CENOPROG);
	break;
    case CEPROCESO:
        std::cerr << "Error en el proceso" << endl;
	exit(CEPROCESO);
	break;
    case CNOACOTADO:
        std::cerr << "Raz no acotada en intervalo" << endl;
	exit(CEPROCESO);
	break;
    default: 
	std::cerr << "Error no catalogado" << endl;
	exit(FALLO);
    } 
    
}
#endif 







