// matriz.cpp
// Implementacion de RMatriz

#include <error.h>
#include <matriz.h>

// Constructor estndar
// Crea una matriz con i filas y j columnas m(i,j)
RMatriz::RMatriz(int i, int j) 
{
  pFilas = i;
  pCols = j;
    if ( i < 1 || j < 1)
	error("Error, la dimensin mnima de la matrix es 1",
	      CERANGO);
    // la dimensin de la matriz es ixj
    pElem = new double[pFilas * pCols];
    if (!pElem)
	error("Imposible crear matriz",CENOMEM);

}

// Constructor de copia
RMatriz::RMatriz(const RMatriz& m) 
{
  pFilas = m.pFilas;
  pCols = m.pCols;
  pElem = new double[pFilas * pCols];
    if (!pElem)
	error("Imposible crear matriz",CENOMEM);

  // Copia cada pElem 
  for (int i = 1; i <= pFilas; i++)
      for (int j = 1; j <= pCols; j++)
	  (*this)(i, j) = m(i, j);
}

// Destructor
RMatriz::~RMatriz() 
{
  delete[] pElem;
}

// Devuelve nmero de filas
const int&  RMatriz::filas() const 
{
  return pFilas;
}
// Devuelve nmero de columnas
const int&  RMatriz::columnas() const 
{
  return pCols;
}

// Devuelve un elemento de la matriz
const double& RMatriz::operator () (int fila, int col) const 
{
    // Comprueba rango
    if (fila < 1 || fila > pFilas)
	error("Error, fila fuera de rango",CERANGO);
    if (col < 1 || col > pCols)
	error("Error, columna fuera de rango",CERANGO);
 
    return pElem[(col - 1) + pCols * (fila - 1)];
}

// Asigna un valor
double& RMatriz::operator () (int fila, int col)  
{
    // Comprueba rango
    if (fila < 1 || fila > pFilas)
	error("Error, fila fuera de rango",CERANGO);
    if (col < 1 || col > pCols)
	error("Error, columna fuera de rango",CERANGO);
 
    return pElem[(col - 1) + pCols * (fila - 1)];
}

//Operadores

// Operacin = ( m1 = m)  
// Cdigo idntico al operador de copia
RMatriz& RMatriz::operator = (const RMatriz& m)
{
  pFilas = m.pFilas;
  pCols = m.pCols;
  pElem = new double[pFilas * pCols];
    if (!pElem)
	error("Imposible crear matriz",CENOMEM);

    // Copia cada pElem 
    for (int i = 1; i <= pFilas; i++)
	for (int j = 1; j <= pCols; j++)
	    (*this)(i, j) = m(i, j);
}


// Operacin += (m1+=m -->m1 = m1 + m)
RMatriz& RMatriz::operator += (const RMatriz& m) 
{
    // Comprueba dimensin
    if (m.pFilas != pFilas || m.pCols != pCols)
	error("Las matrices no tienen la misma dimensin",
	      CEMALDIM);
  
    // Suma cada elemento
    for (int i = 1; i <= pFilas; i++)
	for (int j = 1; j <= pCols; j++)
	    (*this)(i, j) += m(i, j);
  
    return *this;
}

// Operacin -= (m1-=m -->m1 = m1 - m)
RMatriz& RMatriz::operator -= (const RMatriz& m) 
{
    // Comprueba dimensin
    if (m.pFilas != pFilas || m.pCols != pCols)
	error("Las matrices no tienen la misma dimensin",
	      CEMALDIM);
  
    // Resta cada elemento
    for (int i = 1; i <= pFilas; i++)
	for (int j = 1; j <= pCols; j++)
	    (*this)(i, j) -= m(i, j);
  
    return *this;
}

// Operacin *= (m*=a -->m = m * a)
RMatriz& RMatriz::operator *= (double a) 
{
    // Multiplica cada elemento de la matriz por el factor a */ 
    for (int i = 1; i <= pFilas; i++)
	for (int j = 1; j <= pCols; j++)
	    (*this)(i, j) *= a;
    return *this;
}

// Operaciones con matrices

// Cambio de signo
RMatriz operator - (const RMatriz& m1)
{
    // Crea una nueva matriz como una copia de m1
    RMatriz m = m1;   // Uso del operador de asignacin (=)
    for (int i = 1; i <= m.filas(); i++)
	for (int j = 1; j <= m.columnas(); j++)
	m(i,j) = -m1(i,j);   
    return m;
}

// Suma
RMatriz operator + (const RMatriz& m1, const RMatriz& m2) {
    // Comprueba dimensin
    if (m1.filas() != m2.filas() || 
	m1.columnas() != m2.columnas())
	error("Las matrices no tienen la misma dimensin",
	      CEMALDIM);

    // Crea una nueva matriz copia de m1
    RMatriz m(m1);  // m = m1
    // Y suma
    m+=m2;          // m = m+m2 =m1+m2
    return m;
}

// Resta
RMatriz operator - (const RMatriz& m1, const RMatriz& m2) {
    // Comprueba dimensin
    if (m1.filas() != m2.filas() || 
	m1.columnas() != m2.columnas())
	error("Las matrices no tienen la misma dimensin",
	      CEMALDIM);

    // Crea una nueva matriz copia de m1
    RMatriz m(m1);  // m = m1
    // Y resta
    m-=m2;          // m = m+m2 =m1+m2
    return m;
}
// Producto por un escalar
RMatriz operator * (const RMatriz& m1, double a)
{
    // Crea una copia de m1
    RMatriz m(m1);   // Uso del operador de asignacin (=)
    m*=a;            // m *=a --> m = m*a
    return m;
}

RMatriz operator * (double a, const RMatriz& m)
{
    return m*a;  // a* v = v*a
}

// Producto de matrices
RMatriz operator * (const RMatriz& m1, const RMatriz& m2) 
{
    // Comprueba que tienen la dimensin adecuada 
    if (m1.columnas() != m2.filas()) 
	error("Intento de multiplicar matrices con dimensiones incompatibles",
	      CEMALDIM);
  
    // Crea una nueva matriz con la dimensin adecuada
  RMatriz m(m1.filas(), m2.columnas());
  
  // Multiplica elemento a elemento
  for (int i = 1; i <= m1.filas(); i++)
    for (int j = 1; j <= m2.columnas(); j++) {
	double subtotal = 0;
	for (int k = 1; k <= m1.columnas(); k++)
	    subtotal += m1(i, k) * m2(k, j);
      
	m(i, j) = subtotal;
    }
  
  return m;
}

// Matriz traspuesta
RMatriz traspuesta(const RMatriz& m) 
{
    // Crea una matriz de columnas x filas
    RMatriz mt(m.columnas(), m.filas());

    // Copia filas en columnas
    for (int i = 1; i <= mt.filas(); i++)
	for (int j = 1; j <= mt.columnas(); j++)
	    mt(i, j) = m(j, i);

    return mt;
}

// Escribe una matriz
ostream& operator << (ostream& s, const RMatriz& m) 
{
    s << endl;

    for (int i = 1; i <= m.filas(); i++) {
	s << "(\t";
    
	for (int j = 1; j <= m.columnas(); j++)
	    s << m(i, j) << "\t";
    
	s << ")" << endl;
    }
  
    return s;
}
