/* -*- mode: c++ -*- */
// matutil.h
// Declaraciones tiles para manipulaciones matemticas

#ifndef __MATH_H__
#define __MATH_H__
#include <string>
#include <cmath>
#include <sstream>
#include <error.h>
using namespace std;  

// Definimos constantes matemticas de inters
static const double EXP = 2.71828182845904523536028747135 ;    // e 
static const double LOG2E = 1.44269504088896340735992468100;    //log_2 e
static const double LOG10E = 0.43429448190325182765112891892;  // log_10 (e)
static const double SQRT2 = 1.41421356237309504880168872421;   // sqrt(2)
static const double SQRT1_2 =0.70710678118654752440084436210;  // sqrt(1/2)
static const double SQRT3 = 1.73205080756887729352744634151 ;  // sqrt(3)
static const double PI = 3.14159265358979323846264338328  ;    // pi
static const double PI_2 = 1.57079632679489661923132169164 ;   // pi/2
static const double PI_4 = 0.78539816339744830966156608458 ;   // pi/4
static const double SQRTPI = 1.77245385090551602729816748334 ; // sqrt(pi)
static const double LN10 = 2.30258509299404568401799145468 ;    // ln(10)
static const double LN2 = 0.69314718055994530941723212146 ;     // ln(2)
static const double LNPI = 1.14472988584940017414342735135 ;    // ln(pi)
static const double EULER = 0.57721566490153286060651209008 ;   // cte Euler 
static const double AUREA = 0.618033988749895; // Media Area 

// Nmeros grandes y pequeos
static const double BIG = 1e+99;
static const double SMALL = 1e-10;

/**
 *\defgroup util Utilidades de Calnum
 */

/// puntero a una funcin 
/**
 * Toma un valor en doble precisin y devuelve un valor en doble precisin
 *\ingroup util
 */ 
typedef double (*func) (double) ;
/// puntero a una funcin 
/**
 * Toma dos valores en doble precisin y devuelve un valor en doble precisin
 *\ingroup util
 */ 
typedef double (*fun2) (double, double) ;

/// Nivel de informacin proporcionado por la rutina
enum info {NOINFO,CONCISO,DETALLADO};

/// Imagen de una funcin
/** Calcula la imagen de una funcin asegurndose que sta es continua.
 * - devuelve un cdigo de error si la funcin no es continua
 * (utilizando la funcin "finite", definida en <cfloat>). En
 * otro caso devuelve EXITO
 *\ingroup util
 */
inline int f_finite(func F, double a, double& fa)
{
  fa = F(a);     

  // la funcin debe ser continua en a
  if (!finite(fa))
    return CEFUNC;

  return EXITO;
}

/// Algunas funciones templatizadas tiles 
/** 
 *\ingroup util
 */
template <class T> 
inline T signo(T a)
{
    if (a >=0) return 1; 
    else return -1;
}
/// mximo 
/** 
 *\ingroup util
 */
template <class T> 
inline T maximo (T a, T b)
{
    if (a > b) return a; else return b;
}
/// mnimo
/** 
 *\ingroup util
 */
template <class T> 
inline T minimo (T a, T b)
{
    if (a > b) return b; else return a;
}
/// devuelve true (verdad) si la funcin es impar
/** 
 *\ingroup util
 */
template <class T> 
inline bool esImpar(T n)
{
    if (n & 1) return true; 
    else return false;
}
/// devuelve true (verdad) si la funcin es par
/** 
 *\ingroup util
 */
template <class T> 
inline bool esPar(T n)
{
    if (!esImpar(n)) return true; 
    else return false;
}
/// calcula el cuadrado de un nmero
/** 
 *\ingroup util
 */
template <class T> 
inline T cuadrado(T a)
{
    return a*a; 
}
/// calcula el cubo de un nmero
/** 
 *\ingroup util
 */
template <class T> 
inline T cubo(T a)
{
    return cuadrado(a)*a; 
}
/// calcula la cuarta potencia de un nmero
/** 
 *\ingroup util
 */
template <class T> 
inline T pow4(T a)
{
    return cuadrado(a)*cuadrado(a); 
}
/// calcula la quinta potencia de un nmero
/** 
 *\ingroup util
 */
template <class T> 
inline T pow5(T a)
{
    return pow4(a)*a; 
}

/// calcula la sexta potencia de un nmero
/** 
 *\ingroup util
 */
template <class T> 
inline T pow6(T a)
{
    return pow4(a)*cuadrado(a); 
}

/// convierte un nmero a string
/** 
 *\ingroup util
 */
template <class T> 
inline string convierteAString(T i)
{
  ostringstream ostr;
  ostr << i << ends;
  return ostr.str();
}

// Funciones especiales sencillas

#include "specfunc.h"
/// Calcula el factorial de un nmero
/** 
 *\ingroup util
 */

inline double factorial(int n)
{
    if(n <= FACT_TABLE_MAX)
	return fact_table[n].f;
    else
	error("Error en factorial, n demasiado grande",
	      CEOVRFLW);
    return 0;
}

#endif /* __GSL_MATH_H__ */









