// vector.cpp
// Implementacin de la clase RVector
//Modificado por Marc Belda
#include <error.h>
#include <vector.h>

// Constructor estndar. Crea un vector de dimensin n
RVector::RVector(int n) 
{
    if (n < 1)
	error("Error, la dimensin mnima del vector es 1",CERANGO);
    pDim = n;
    pElem = new double[pDim];
    if (!pElem)
	error("Imposible crear vector",CENOMEM);
}
// Constructor de copia
RVector::RVector(const RVector& v) 
{
    // Crea un nuevo vector con la dimensin de v  
    pDim = v.pDim;
    pElem = new double[pDim];
    if (!pElem)
	error("Imposible crear vector",CENOMEM);

    // Y copia cada elemento
    for (int i = 1; i <= pDim; i++)
      (*this)(i) = v(i);
}

// Destructor
RVector::~RVector() 
{
    delete[] pElem;
}

// Funciones de acceso

// devuelve la dimensin del vector
const int& RVector::dimension() const
{
    return pDim;
}

// Operadores

// Devuelve el elemento v(i), i=1,2...n
const double& RVector::operator () (int i) const
{
    // Comprueba el rango
    if (i < 1 || i > pDim)
	error("Error, indice fuera de rango",CERANGO);
  
    return pElem[i - 1];

}
// Asigna el elemento v(i), i=1,2...n
double& RVector::operator () (int i)
{
    // Comprueba el rango
    if (i < 1 || i > pDim)
	error("Error, indice fuera de rango",CERANGO);
  
    return pElem[i - 1];

}

// Operacin = ( v1 = v)  
RVector& RVector::operator = (const RVector& v)
{
    // Crea un nuevo vector con la dimensin de v  
    pDim = v.pDim;
    pElem = new double[pDim];
    if (!pElem)
	error("Imposible crear vector",CENOMEM);

    // Y copia cada elemento
    for (int i = 1; i <= pDim; i++)
	(*this)(i) = v(i);
    return *this;
}

// Operacin += ( v+=v1 ==> v = v +v1)  
RVector& RVector::operator += (const RVector& v1)
{
    // v y v1 deben tener la misma dimensin
    if(pDim != v1.pDim)
	error("Los vectores no tienen la misma dimensin",CEMALDIM);
  
    // Suma los elementos
    for (int i = 1; i <= pDim; i++)
	(*this)(i) += v1(i);
    return *this;
}

// Operacin -= ( v -=v1 ==> v = v - v1)  
RVector& RVector::operator -= (const RVector& v1)
{
    // v y v1 deben tener la misma dimensin
    if(pDim != v1.pDim)
	error("Los vectores no tienen la misma dimensin",CEMALDIM);
  
    // Resta los elementos
    for (int i = 1; i <= pDim; i++)
	(*this)(i) -= v1(i);
    return *this;

}

// Operacin*= ( v*= a --> v = v*a, a es un real)
RVector& RVector::operator *= (double a)
{
    for (int i = 1; i <= pDim; i++)
	(*this)(i) *= a;
    return *this;
}
//Definicion de la funcion declarada interna a la clase,funcion intercambia!!
RVector& RVector::intercambia(int i,int j)
{
if (i < 1 ||i > pDim)
error("Error,indice fuera de rango",CERANGO);
double temp=0;
temp=(*this)(i);
(*this)(i)=(*this)(j);
(*this)(j)=temp;
return *this;
}
// Operaciones con vectores
// v= -v1
RVector operator - (const RVector& v1)
{
    // Crea un nuevo vector como una copia de v1
    RVector v = v1;   // Uso del operador de asignacin (=)
    for (int i = 1; i <= v.dimension(); i++)
	v(i) = -v1(i);   
    return v;
}

// v= v1 + v2
RVector operator + (const RVector& v1, const RVector& v2)
{
    // Crea un nuevo vector como una copia de v1
    RVector v = v1;   // Uso del operador de asignacin (=)
    v+=v2;            // v = v + v2 --> v1 + v2
    return v;
}

// v= v1 - v2
RVector operator - (const RVector& v1, const RVector& v2)
{
    // Crea un nuevo vector como una copia de v1
    RVector v = v1;   // Uso del operador de asignacin (=)
    v-=v2;            // v = v - v2 --> v1 - v2
    return v;
}

// Producto escalar (sobrecarga operador *)
double operator * (const RVector& v1, const RVector& v2)
{
    return productoEscalar(v1,v2);
}
// v = v1*a
RVector operator * (const RVector& v1, double a)
{
    // Crea un nuevo vector como una copia de v1
    RVector v = v1;   // Uso del operador de asignacin (=)
    v*=a;            // v *=a --> v = v*a
    return v;

}
RVector operator * (double a, const RVector& v)
{
    return v*a;  // a* v = v*a
}

double productoEscalar(const RVector& v1, const RVector& v2)
{
    // v1 y v2 deben tener la misma dimensin
    if(v1.dimension() != v2.dimension()) 
	error("Los vectores no tienen la misma dimensin",CEMALDIM);

    double escalar = 0;
    for (int i = 1; i <= v1.dimension(); i++)
	escalar += v1(i)*v2(i);
    return escalar;
}
ostream& operator << (ostream& s, const RVector& v)
{
  s << endl; 
  s << "(\t";
  for (int i = 1; i <= v.dimension(); i++) 
      s << v(i) << "\t";
    
  s << ")" << endl;
  return s;
}
//Definicion de la funciones declaradas externas a la clase,funcion norma,maximo y minimo!!
double norma(const RVector& v1)
{
double z;
double norma=0;
for (int i=1; i<=v1.dimension();i++)
{
norma+=v1(i)*v1(i);
}
z=sqrt(norma);
return z;
}
int maximo(const RVector& v1,double& max)
{
int imax=1;
max=v1(1);
for (int i=2;i<=v1.dimension();i++)
{
if (v1(i)>max)
{
max=v1(i);
imax=i;
}
}
return imax;
}
int minimo(const RVector& v1,double& min)
{
int imin=1;
min=v1(1);
for (int i=2;i<=v1.dimension();i++)
{
if (v1(i)<min)
{
min=v1(i);
imin=i;
}
}
return imin;
}








